import { isReactive, toRefs, ref, readonly, reactive, computed } from 'vue';

/**
 * @public
 */
function getOwnKeys(object) {
    return Array.from(Object.keys(object));
}
/**
 * @public
 */
function assert(condition, message) {
    if (!condition)
        throw Error("[vuex-light]: " + message);
}
/**
 * @public
 */
function isPlainObject(object) {
    return Object.prototype.toString.call(object) === '[object Object]';
}

/**
 * @public
 */
function createStore(stateOption, gettersOption, mutationsOption, actionsOption, pluginsOption, modules) {
    if (modules === void 0) { modules = {}; }
    if ((process.env.NODE_ENV !== 'production')) {
        assert(isPlainObject(stateOption), 'invalid state type.');
    }
    var _a = useSubscriber(), subscribe = _a.subscribe, subscribers = _a.subscribers;
    var _b = useSubscriber(), subscribeAction = _b.subscribe, actionSubscribers = _b.subscribers;
    var normalizedStateOption = isReactive(stateOption) ? toRefs(stateOption) : stateOption;
    var state = ref(getOwnKeys(normalizedStateOption).reduce(function (state, stateKey) {
        var _a;
        return Object.assign(state, (_a = {}, _a[stateKey] = normalizedStateOption[stateKey], _a));
    }, {}));
    var normalizedGettersOption = gettersOption || {};
    var getters = readonly(reactive(getOwnKeys(normalizedGettersOption).reduce(function (rawGetters, getterKey) {
        var _a;
        var getter = computed(function () {
            return normalizedGettersOption[getterKey]({ state: state.value, getters: getters });
        });
        return Object.assign(rawGetters, (_a = {}, _a[getterKey] = getter, _a));
    }, {})));
    var normalizedMutationsOption = mutationsOption || {};
    var mutations = getOwnKeys(normalizedMutationsOption).reduce(function (mutations, mutationKey) {
        var _a;
        var mutation = function (payload) {
            var result = normalizedMutationsOption[mutationKey]({ state: state.value, getters: getters, mutations: mutations }, payload);
            subscribers.forEach(function (subscriber) { return subscriber.call(null, { key: mutationKey, payload: payload }); });
            return result;
        };
        return Object.assign(mutations, (_a = {}, _a[mutationKey] = mutation, _a));
    }, {});
    var normalizedActionsOption = actionsOption || {};
    var actions = getOwnKeys(normalizedActionsOption).reduce(function (actions, actionKey) {
        var _a;
        var action = function (payload) {
            actionSubscribers.forEach(function (subscriber) { return subscriber.call(null, { key: actionKey, payload: payload }); });
            return normalizedActionsOption[actionKey]({
                state: state.value,
                getters: getters,
                mutations: mutations,
                actions: actions,
            }, payload);
        };
        return Object.assign(actions, (_a = {}, _a[actionKey] = action, _a));
    }, {});
    var replaceState = function (newState) {
        getOwnKeys(state.value)
            .filter(function (key) { return newState[key] === undefined; })
            .forEach(function (key) { return delete state.value[key]; });
        getOwnKeys(newState).forEach(function (key) {
            state.value[key] = newState[key];
        });
    };
    var store = {
        state: state.value,
        getters: getters,
        mutations: mutations,
        actions: actions,
        subscribe: subscribe,
        actionSubscribe: function (subscriber) {
            console.warn('Deprecated: use `subscribeAction` instead.');
            return subscribeAction(subscriber);
        },
        subscribeAction: subscribeAction,
        replaceState: replaceState,
        modules: modules,
    };
    var optionPlugins = pluginsOption || [];
    optionPlugins.forEach(function (plugin) { return plugin(store); });
    return store;
}
/**
 * @public
 */
function useSubscriber() {
    var subscribers = [];
    var subscribe = function (subscriber) {
        subscribers.push(subscriber);
    };
    return {
        subscribe: subscribe,
        subscribers: subscribers,
    };
}

function t(t,r,e){return void 0===(t=(r.split?r.split("."):r).reduce(function(t,r){return t&&t[r]},t))?e:t}function r(t,r,e,n){return !/^(__proto__|constructor|prototype)$/.test(r)&&((r=r.split?r.split("."):r.slice(0)).slice(0,-1).reduce(function(t,r){return t[r]=t[r]||{}},t)[r.pop()]=e),t}

function _isPlaceholder(a) {
  return a != null && typeof a === 'object' && a['@@functional/placeholder'] === true;
}

/**
 * Optimized internal one-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry1(fn) {
  return function f1(a) {
    if (arguments.length === 0 || _isPlaceholder(a)) {
      return f1;
    } else {
      return fn.apply(this, arguments);
    }
  };
}

/**
 * Optimized internal two-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry2(fn) {
  return function f2(a, b) {
    switch (arguments.length) {
      case 0:
        return f2;

      case 1:
        return _isPlaceholder(a) ? f2 : _curry1(function (_b) {
          return fn(a, _b);
        });

      default:
        return _isPlaceholder(a) && _isPlaceholder(b) ? f2 : _isPlaceholder(a) ? _curry1(function (_a) {
          return fn(_a, b);
        }) : _isPlaceholder(b) ? _curry1(function (_b) {
          return fn(a, _b);
        }) : fn(a, b);
    }
  };
}

/**
 * Optimized internal three-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry3(fn) {
  return function f3(a, b, c) {
    switch (arguments.length) {
      case 0:
        return f3;

      case 1:
        return _isPlaceholder(a) ? f3 : _curry2(function (_b, _c) {
          return fn(a, _b, _c);
        });

      case 2:
        return _isPlaceholder(a) && _isPlaceholder(b) ? f3 : _isPlaceholder(a) ? _curry2(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder(b) ? _curry2(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _curry1(function (_c) {
          return fn(a, b, _c);
        });

      default:
        return _isPlaceholder(a) && _isPlaceholder(b) && _isPlaceholder(c) ? f3 : _isPlaceholder(a) && _isPlaceholder(b) ? _curry2(function (_a, _b) {
          return fn(_a, _b, c);
        }) : _isPlaceholder(a) && _isPlaceholder(c) ? _curry2(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder(b) && _isPlaceholder(c) ? _curry2(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _isPlaceholder(a) ? _curry1(function (_a) {
          return fn(_a, b, c);
        }) : _isPlaceholder(b) ? _curry1(function (_b) {
          return fn(a, _b, c);
        }) : _isPlaceholder(c) ? _curry1(function (_c) {
          return fn(a, b, _c);
        }) : fn(a, b, c);
    }
  };
}

function _has(prop, obj) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}

function _cloneRegExp(pattern) {
  return new RegExp(pattern.source, (pattern.global ? 'g' : '') + (pattern.ignoreCase ? 'i' : '') + (pattern.multiline ? 'm' : '') + (pattern.sticky ? 'y' : '') + (pattern.unicode ? 'u' : ''));
}

/**
 * Gives a single-word string description of the (native) type of a value,
 * returning such answers as 'Object', 'Number', 'Array', or 'Null'. Does not
 * attempt to distinguish user Object types any further, reporting them all as
 * 'Object'.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Type
 * @sig (* -> {*}) -> String
 * @param {*} val The value to test
 * @return {String}
 * @example
 *
 *      R.type({}); //=> "Object"
 *      R.type(1); //=> "Number"
 *      R.type(false); //=> "Boolean"
 *      R.type('s'); //=> "String"
 *      R.type(null); //=> "Null"
 *      R.type([]); //=> "Array"
 *      R.type(/[A-z]/); //=> "RegExp"
 *      R.type(() => {}); //=> "Function"
 *      R.type(undefined); //=> "Undefined"
 */

var type =
/*#__PURE__*/
_curry1(function type(val) {
  return val === null ? 'Null' : val === undefined ? 'Undefined' : Object.prototype.toString.call(val).slice(8, -1);
});

/**
 * Copies an object.
 *
 * @private
 * @param {*} value The value to be copied
 * @param {Array} refFrom Array containing the source references
 * @param {Array} refTo Array containing the copied source references
 * @param {Boolean} deep Whether or not to perform deep cloning.
 * @return {*} The copied value.
 */

function _clone(value, refFrom, refTo, deep) {
  var copy = function copy(copiedValue) {
    var len = refFrom.length;
    var idx = 0;

    while (idx < len) {
      if (value === refFrom[idx]) {
        return refTo[idx];
      }

      idx += 1;
    }

    refFrom[idx + 1] = value;
    refTo[idx + 1] = copiedValue;

    for (var key in value) {
      copiedValue[key] = deep ? _clone(value[key], refFrom, refTo, true) : value[key];
    }

    return copiedValue;
  };

  switch (type(value)) {
    case 'Object':
      return copy({});

    case 'Array':
      return copy([]);

    case 'Date':
      return new Date(value.valueOf());

    case 'RegExp':
      return _cloneRegExp(value);

    default:
      return value;
  }
}

/**
 * Creates a deep copy of the value which may contain (nested) `Array`s and
 * `Object`s, `Number`s, `String`s, `Boolean`s and `Date`s. `Function`s are
 * assigned by reference rather than copied
 *
 * Dispatches to a `clone` method if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {*} -> {*}
 * @param {*} value The object or array to clone
 * @return {*} A deeply cloned copy of `val`
 * @example
 *
 *      const objects = [{}, {}, {}];
 *      const objectsClone = R.clone(objects);
 *      objects === objectsClone; //=> false
 *      objects[0] === objectsClone[0]; //=> false
 */

var clone =
/*#__PURE__*/
_curry1(function clone(value) {
  return value != null && typeof value.clone === 'function' ? value.clone() : _clone(value, [], [], true);
});

function _isObject(x) {
  return Object.prototype.toString.call(x) === '[object Object]';
}

/**
 * Creates a new object with the own properties of the two provided objects. If
 * a key exists in both objects, the provided function is applied to the key
 * and the values associated with the key in each object, with the result being
 * used as the value associated with the key in the returned object.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Object
 * @sig ((String, a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeDeepWithKey, R.merge, R.mergeWith
 * @example
 *
 *      let concatValues = (k, l, r) => k == 'values' ? R.concat(l, r) : r
 *      R.mergeWithKey(concatValues,
 *                     { a: true, thing: 'foo', values: [10, 20] },
 *                     { b: true, thing: 'bar', values: [15, 35] });
 *      //=> { a: true, b: true, thing: 'bar', values: [10, 20, 15, 35] }
 * @symb R.mergeWithKey(f, { x: 1, y: 2 }, { y: 5, z: 3 }) = { x: 1, y: f('y', 2, 5), z: 3 }
 */

var mergeWithKey =
/*#__PURE__*/
_curry3(function mergeWithKey(fn, l, r) {
  var result = {};
  var k;

  for (k in l) {
    if (_has(k, l)) {
      result[k] = _has(k, r) ? fn(k, l[k], r[k]) : l[k];
    }
  }

  for (k in r) {
    if (_has(k, r) && !_has(k, result)) {
      result[k] = r[k];
    }
  }

  return result;
});

/**
 * Creates a new object with the own properties of the two provided objects.
 * If a key exists in both objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to the key and associated values
 *   using the resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key
 * of the resulting object.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig ((String, a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.mergeWithKey, R.mergeDeepWith
 * @example
 *
 *      let concatValues = (k, l, r) => k == 'values' ? R.concat(l, r) : r
 *      R.mergeDeepWithKey(concatValues,
 *                         { a: true, c: { thing: 'foo', values: [10, 20] }},
 *                         { b: true, c: { thing: 'bar', values: [15, 35] }});
 *      //=> { a: true, b: true, c: { thing: 'bar', values: [10, 20, 15, 35] }}
 */

var mergeDeepWithKey =
/*#__PURE__*/
_curry3(function mergeDeepWithKey(fn, lObj, rObj) {
  return mergeWithKey(function (k, lVal, rVal) {
    if (_isObject(lVal) && _isObject(rVal)) {
      return mergeDeepWithKey(fn, lVal, rVal);
    } else {
      return fn(k, lVal, rVal);
    }
  }, lObj, rObj);
});

/**
 * @public
 */
function defaultReducer(state, paths) {
    var unwrappedState = Object.keys(state).reduce(function (unwrappedState, key) {
        unwrappedState[key] = state[key];
        return unwrappedState;
    }, {});
    return paths === null
        ? unwrappedState
        : paths.reduce(function (substate, path) { return r(substate, path, t(unwrappedState, path)); }, {});
}
/**
 * @public
 */
function defaultSubscriber(store) {
    return function (handler) {
        return store.subscribe(handler);
    };
}
/**
 * @public
 */
function defaultGetState(key, storage) {
    var value = storage.getItem(key);
    try {
        return typeof value !== 'undefined' ? JSON.parse(value) : undefined;
    }
    catch (err) { }
    return undefined;
}
/**
 * @public
 */
function defaultSetState(key, state, storage) {
    return storage.setItem(key, JSON.stringify(state));
}
var randomKey = '4u33j8eqxxyndzw5slhe6xyt9tuymkvf';
/**
 * @public
 */
function defaultAssertStorage(storage) {
    storage.setItem(randomKey, 1);
    storage.removeItem(randomKey);
}
function normalize$1(options) {
    return {
        key: options.key || 'vuex',
        paths: options.paths || null,
        reducer: options.reducer || defaultReducer,
        subscriber: options.subscriber || defaultSubscriber,
        storage: options.storage || window.localStorage,
        getState: options.getState || defaultGetState,
        setState: options.setState || defaultSetState,
        filter: options.filter || (function () { return true; }),
        overwrite: options.overwrite !== undefined ? options.overwrite : false,
        fetchBeforeUse: options.fetchBeforeUse !== undefined ? options.fetchBeforeUse : false,
        assertStorage: options.assertStorage || defaultAssertStorage,
        mergeDeepWithKeyFn: options.mergeDeepWithKeyFn || (function (_k, _l, r) { return r; }),
        onRehydrated: options.onRehydrated || (function () { }),
    };
}
/**
 * @public
 */
function createPersistPlugin(options) {
    var normalizedOptions = normalize$1(options || {});
    normalizedOptions.assertStorage(normalizedOptions.storage);
    var savedState;
    if (normalizedOptions.fetchBeforeUse) {
        savedState = normalizedOptions.getState(normalizedOptions.key, normalizedOptions.storage);
    }
    // TODO: add type info
    return function (store) {
        if (normalizedOptions.fetchBeforeUse === false) {
            savedState = normalizedOptions.getState(normalizedOptions.key, normalizedOptions.storage);
        }
        if (typeof savedState === 'object' && savedState !== null) {
            store.replaceState(normalizedOptions.overwrite
                ? savedState
                : mergeDeepWithKey(normalizedOptions.mergeDeepWithKeyFn, store.state, savedState));
            normalizedOptions.onRehydrated(store);
        }
        normalizedOptions.subscriber(store)(function (mutation) {
            if (normalizedOptions.filter(mutation.key))
                normalizedOptions.setState(normalizedOptions.key, normalizedOptions.reducer(store['state'], normalizedOptions.paths), normalizedOptions.storage);
        });
    };
}

/**
 * @public
 */
var defaultCollapsed = true;
/**
 * @public
 */
function defaultFilter(_mutation, _prevState, _state) {
    return true;
}
/**
 * @public
 */
function defaultActionFilter(_action) {
    return true;
}
/**
 * @public
 */
function defaultTransformer(state) {
    return state;
}
/**
 * @public
 */
function defaultMutationTransformer(mutation) {
    return mutation;
}
/**
 * @public
 */
function defaultActionTransformer(action) {
    return action;
}
/**
 * @public
 */
var defaultLogger = console;
function normalize(options) {
    return {
        collapsed: options.collapsed !== undefined ? options.collapsed : defaultCollapsed,
        filter: options.filter || defaultFilter,
        actionFilter: options.actionFilter || defaultActionFilter,
        transformer: options.transformer || defaultTransformer,
        mutationTransformer: options.mutationTransformer || defaultMutationTransformer,
        actionTransformer: options.actionTransformer || defaultActionTransformer,
        logMutations: options.logMutations !== undefined ? options.logMutations : true,
        logActions: options.logActions !== undefined ? options.logActions : true,
        logger: options.logger || defaultLogger,
    };
}
/**
 * @public
 */
function createLoggerPlugin(options) {
    var _a = normalize(options || {}), collapsed = _a.collapsed, filter = _a.filter, actionFilter = _a.actionFilter, transformer = _a.transformer, mutationTransformer = _a.mutationTransformer, actionTransformer = _a.actionTransformer, logMutations = _a.logMutations, logActions = _a.logActions, logger = _a.logger;
    // TODO: add type info
    return function (store) {
        var prevState = clone(store.state);
        if (logMutations) {
            // TODO: add type info
            store.subscribe(function (mutation) {
                var state = clone(store.state);
                if (filter(mutation, prevState, state)) {
                    var formattedTime = getFormattedTime();
                    var formattedMutation = mutationTransformer(mutation);
                    var message = "mutation " + mutation.key + formattedTime;
                    startMessage(logger, message, collapsed);
                    logger.log('%c prev state', 'color: #9E9E9E; font-weight: bold', transformer(prevState));
                    logger.log('%c mutation', 'color: #03A9F4; font-weight: bold', formattedMutation);
                    logger.log('%c next state', 'color: #4CAF50; font-weight: bold', transformer(state));
                    endMessage(logger);
                }
                prevState = state;
            });
        }
        if (logActions) {
            // TODO: add type info
            store.subscribeAction(function (action) {
                if (actionFilter(action) === false)
                    return;
                var formattedTime = getFormattedTime();
                var formattedAction = actionTransformer(action);
                var message = "action " + action.key + formattedTime;
                startMessage(logger, message, collapsed);
                logger.log('%c action', 'color: #03A9F4; font-weight: bold', formattedAction);
                endMessage(logger);
            });
        }
    };
}
function startMessage(logger, message, collapsed) {
    var startMessage = collapsed ? logger.groupCollapsed : logger.group;
    try {
        startMessage.call(logger, message);
    }
    catch (e) {
        logger.log(message);
    }
}
function endMessage(logger) {
    try {
        logger.groupEnd();
    }
    catch (e) {
        logger.log('—— log end ——');
    }
}
function getFormattedTime() {
    var time = new Date();
    return " @ " + time.getHours().toString().padStart(2, '0') + ":" + time.getMinutes().toString().padStart(2, '0') + ":" + time
        .getSeconds()
        .toString()
        .padStart(2, '0') + "." + time.getMilliseconds().toString().padStart(2, '0');
}

export { assert, createLoggerPlugin, createPersistPlugin, createStore, defaultActionFilter, defaultActionTransformer, defaultAssertStorage, defaultCollapsed, defaultFilter, defaultGetState, defaultLogger, defaultMutationTransformer, defaultReducer, defaultSetState, defaultSubscriber, defaultTransformer, getOwnKeys, isPlainObject, useSubscriber };
