import { OmitFirstParameter, DeepReadonly, AsyncFunc } from './util';
/**
 * @public
 */
export declare type Key = string | number;
/**
 * @public
 */
export declare type Payload = any;
/**
 * @public
 */
export declare type StateOptionType = Record<Key, any>;
/**
 * @public
 */
export declare type GettersOptionType<StateOption extends StateOptionType> = Record<Key, ({ state, getters }: {
    state: DeepReadonly<StateOption>;
    getters: any;
}) => any>;
/**
 * @public
 */
export declare type MutationsOptionType<StateOption extends StateOptionType, GettersOption extends GettersOptionType<StateOption>> = Record<Key, Exclude<({ state, getters, mutations, }: {
    state: StateOption;
    getters: DeepReadonly<{
        [P in keyof GettersOption]: ReturnType<GettersOption[P]>;
    }>;
    mutations: any;
}, ...payload: Payload[]) => void, AsyncFunc>>;
/**
 * @public
 */
export declare type ActionsOptionType<StateOption extends StateOptionType, GettersOption extends GettersOptionType<StateOption>, MutationsOption extends MutationsOptionType<StateOption, GettersOption>> = Record<Key, ({ state, getters, mutations, actions, }: {
    state: DeepReadonly<StateOption>;
    getters: DeepReadonly<{
        [P in keyof GettersOption]: ReturnType<OmitFirstParameter<GettersOption[P]>>;
    }>;
    mutations: DeepReadonly<{
        [P in keyof MutationsOption]: OmitFirstParameter<MutationsOption[P]>;
    }>;
    actions: any;
}, ...payload: Payload[]) => void>;
/**
 * @public
 */
export declare type Plugin = (store: ReturnType<typeof createStore>) => void;
/**
 * @public
 */
export declare type Subscriber = (mutation: {
    key: string;
    payload: unknown;
}) => void;
/**
 * @public
 */
export declare type ActionSubscriber = (action: {
    key: string;
    payload: unknown;
}) => void;
/**
 * @public
 */
export declare type CreateStoreReturnType = ReturnType<typeof createStore>;
/**
 * @public
 */
export declare function createStore<StateOption extends StateOptionType, GettersOption extends GettersOptionType<StateOption>, MutationsOption extends MutationsOptionType<StateOption, GettersOption>, ActionsOption extends ActionsOptionType<StateOption, GettersOption, MutationsOption>, Modules extends {
    [P: string]: any;
}>(stateOption: StateOption, gettersOption?: GettersOption, mutationsOption?: MutationsOption, actionsOption?: ActionsOption, pluginsOption?: Plugin[], modules?: Modules): {
    state: DeepReadonly<StateOption>;
    getters: { readonly [P in keyof GettersOption]: DeepReadonly<ReturnType<OmitFirstParameter<GettersOption[P]>>>; };
    mutations: { readonly [P_1 in keyof MutationsOption]: Exclude<OmitFirstParameter<MutationsOption[P_1]>, AsyncFunc>; };
    actions: { readonly [P_2 in keyof ActionsOption]: OmitFirstParameter<ActionsOption[P_2]>; };
    subscribe: (subscriber: Subscriber) => void;
    actionSubscribe: (subscriber: ActionSubscriber) => void;
    subscribeAction: (subscriber: ActionSubscriber) => void;
    replaceState: (newState: StateOption) => void;
    modules: Modules;
};
/**
 * @public
 */
export declare function useSubscriber<Subscriber>(): {
    subscribe: (subscriber: Subscriber) => void;
    subscribers: Subscriber[];
};
