import { Subscriber, CreateStoreReturnType } from '../../';
import { DeepReadonly } from '../../util';
/**
 * @public
 */
export declare function defaultReducer<State extends DeepReadonly<Record<any, any>>>(state: State, paths: string[] | null): any;
/**
 * @public
 */
export declare function defaultSubscriber<Store extends CreateStoreReturnType>(store: Store): (handler: Subscriber) => void;
/**
 * @public
 */
export declare type Storage = {
    getItem: (key: string) => any;
    setItem: (key: string, value: any) => void;
    removeItem: (key: string) => void;
};
/**
 * @public
 */
export declare function defaultGetState(key: Parameters<Storage['getItem']>['0'], storage: Storage): any;
/**
 * @public
 */
export declare function defaultSetState(key: Parameters<Storage['setItem']>['0'], state: any, storage: Storage): void;
/**
 * @public
 */
export declare function defaultAssertStorage(storage: Storage): void | Error;
/**
 * @public
 */
export declare type PersistPluginRequiredOptions = {
    key: string;
    paths: string[] | null;
    reducer: typeof defaultReducer;
    subscriber: typeof defaultSubscriber;
    storage: Storage;
    getState: typeof defaultGetState;
    setState: typeof defaultSetState;
    filter: (mutation: string) => boolean;
    overwrite: boolean;
    fetchBeforeUse: boolean;
    assertStorage: typeof defaultAssertStorage;
    mergeDeepWithKeyFn: (k: string, l: any, r: any) => any;
    onRehydrated: <Store extends CreateStoreReturnType>(_: Store) => void;
};
/**
 * @public
 */
export declare type PersistPluginOptions = Partial<PersistPluginRequiredOptions>;
/**
 * @public
 */
export declare function createPersistPlugin(options?: PersistPluginOptions): (store: any) => void;
